@@ -1,4 +1,9 @@
-#include <Rcpp.h>
+/* !!revision!!
+improvements necessary:
+names for states: prey and predator;
+names for parameter: death_rate_prey etc.; Use readable names
+*/
+#include "header.hpp"
 
 // [[Rcpp::export]]
 Rcpp::NumericVector ode_example(double t, std::vector<double> params, Rcpp::NumericVector y) {
